package com.halo.halojuhesdk.core.base;

import android.content.Context;

import com.halo.halojuhesdk.core.base.utils.LogUtils;


/**
 * Created by dream on 2016/11/10.
 */

public abstract class AbstractProcessor<T extends AbstractTask, P> implements ITaskListener<P> {
    private static final String TAG = "AbstractProcessor";
    private HjTaskManager mTaskManager;

    /**
     * 添加管理器
     *
     * @param manager
     */
    public void setTaskManager(HjTaskManager manager) {
        mTaskManager = manager;
    }

    /**
     * 创建执行Task
     *
     * @return
     */
    protected abstract T createTask();


    /**
     * 执行当前任务
     *
     * @param hjabstracktask
     */
    private void doProcess(T hjabstracktask) {
        LogUtils.d( hjabstracktask.getName() + "  start process");
        // 默认是自启动模式
        if (mTaskManager == null) {
            startBySelf(hjabstracktask);
        } else {
            startByTaskManager(hjabstracktask);
        }
    }

    private void startByTaskManager(T hjabstracktask) {
        if (mTaskManager != null) {
            mTaskManager.add(hjabstracktask);
        }
    }

    private void startBySelf(T hjabstracktask) {
        Thread thread = new Thread(hjabstracktask);
        thread.start();
    }

    /**
     * 预处理在主线程判断是否需要执行
     *
     * @param hjabstracktask
     * @return boolean
     */
    protected boolean canProcess(T hjabstracktask) {
        if (hjabstracktask.preExecute()) {
            LogUtils.d( hjabstracktask.getName() + "  can process");
            return true;
        }
        return false;
    }

    /**
     * 执行完成后处理需要回到原有线程问题
     * @param hjabstracktask
     */
    protected void afterProcess(T hjabstracktask) {
        hjabstracktask.afterExcute();
    }

    /**
     * 开始执行
     */
    public void process() {
        T t = createTask();
        t.setListener(this);
        if (t != null && canProcess(t)) {
            doProcess(t);
            afterProcess(t);
        }
    }
}
