package com.halo.halojuhesdk.core.channel.applovin;

import android.app.Activity;
import android.text.TextUtils;
import android.util.Log;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.MaxReward;
import com.applovin.mediation.MaxRewardedAdListener;
import com.applovin.mediation.ads.MaxRewardedAd;
import com.halo.halojuhesdk.core.base.utils.CompareEcpmUtils;
import com.halo.halojuhesdk.core.channel.IVideoProxyListener;
import com.halo.halojuhesdk.core.channel.IVideoAd;
import com.halo.halojuhesdk.core.request.EcpmCompareBean;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;

public class AppLovinReward implements IVideoAd, MaxRewardedAdListener {
    private MaxRewardedAd rewardedAd;
    private SlotBean mLovinBean;
    private IVideoProxyListener mVideoListener;
    private boolean adLoaded;
    private boolean isShow = true;
    private boolean isClick = false;
    public AppLovinReward(Activity activity, SlotBean bean, final IVideoProxyListener listener) {
        this.mVideoListener = listener;
        this.mLovinBean = bean;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get("max");
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initAppLovin(activity);
                HlAdClient.initSuccessMap.put("max", true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        rewardedAd = MaxRewardedAd.getInstance(bean.adSlotId, activity);
        rewardedAd.setListener(this);
    }


    @Override
    public void loadAd() {
        isShow = true;
        isClick = false;
        rewardedAd.loadAd();
    }

    @Override
    public void showAd() {
        rewardedAd.showAd();
    }

    @Override
    public void onUserRewarded(MaxAd maxAd, MaxReward maxReward) {
        if (mVideoListener != null) {
            mVideoListener.onRewardArrived();
        }
    }

    @Override
    public void onRewardedVideoStarted(MaxAd maxAd) {
        HlStatisManager.getInstance().reportStatic(mLovinBean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_VIDEO_START, mLovinBean.getReqId());
    }

    @Override
    public void onRewardedVideoCompleted(MaxAd maxAd) {
        HlStatisManager.getInstance().reportStatic(mLovinBean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_VIDEO_END, mLovinBean.getReqId());
    }

    @Override
    public void onAdLoaded(MaxAd maxAd) {
        double ecpm = maxAd.getRevenue()*1000;
        Log.e("11111","====videoEcpm===" + ecpm + "====type===" + maxAd.getNetworkName());
        mLovinBean.setEp(ecpm);
        EcpmCompareBean bean = CompareEcpmUtils.compareEcpm(mLovinBean, ecpm);
        mLovinBean.setOp(bean.getEcpm());
        if (bean.getResult()) {
            mVideoListener.onSuccess(InitDataProcessor.SDK_MAX, mLovinBean, bean.getEcpm());
        } else {
            mVideoListener.onAdFailed("applovin: 竞价失败", 102, InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayed(MaxAd maxAd) {
        if (mVideoListener != null) {
            if (isShow) {
                isShow = false;
                mVideoListener.onAdShow(mLovinBean);
            }
        }
    }

    @Override
    public void onAdHidden(MaxAd maxAd) {
        if (mVideoListener != null) {
            mVideoListener.onAdClose();
        }
    }

    @Override
    public void onAdClicked(MaxAd maxAd) {
        if (!isClick) {
            isClick = true;
            if (mVideoListener != null) {
                mVideoListener.onAdClick(mLovinBean);
            }
        }
    }

    @Override
    public void onAdLoadFailed(String s, MaxError maxError) {
        if (mVideoListener != null) {
            mVideoListener.onAdFailed("lovin:error===" + maxError.getMessage(), maxError.getCode(), InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }

    @Override
    public void onAdDisplayFailed(MaxAd maxAd, MaxError maxError) {
        if (mVideoListener != null) {
            mVideoListener.onAdFailed("lovin:error===" + maxError.getMessage(), maxError.getCode(), InitDataProcessor.SDK_MAX, mLovinBean);
        }
    }
}
