package com.halo.halojuhesdk.core.channel.applovin;

import android.app.Activity;
import android.net.Uri;
import android.util.Log;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import com.applovin.mediation.MaxAd;
import com.applovin.mediation.MaxAdRevenueListener;
import com.applovin.mediation.MaxError;
import com.applovin.mediation.nativeAds.MaxNativeAdListener;
import com.applovin.mediation.nativeAds.MaxNativeAdLoader;
import com.applovin.mediation.nativeAds.MaxNativeAdView;
import com.applovin.mediation.nativeAds.MaxNativeAdViewBinder;
import com.applovin.sdk.AppLovinSdkUtils;
import com.halo.halojuhesdk.core.base.utils.CompareEcpmUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.channel.INativeAd;
import com.halo.halojuhesdk.core.channel.INativeProxyListener;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeAd;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeProxyListener;
import com.halo.halojuhesdk.core.common.utils.PxUtil;
import com.halo.halojuhesdk.core.request.EcpmCompareBean;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halo.halojuhesdk.sdk.HlUnifiedAdParameter;

public class AppLovinUnifiedNative extends MaxNativeAdListener implements INativeAd {
    private final HlUnifiedAdParameter mParameter;
    private final Activity mActivity;
    private MaxNativeAdLoader nativeAdLoader;
    private SlotBean mLovinBean;
    private MaxNativeAdView nativeAdView;
    private INativeProxyListener mNativeListener;
    private boolean isShow = true;
    private boolean isClick = false;
    private MaxNativeAdView mNativeView;
    private MaxAd mMaxAd;

    public AppLovinUnifiedNative(Activity activity, SlotBean bean, HlUnifiedAdParameter parameter, final INativeProxyListener listener) {
        mActivity = activity;
        this.mNativeListener = listener;
        this.mLovinBean = bean;
        mParameter = parameter;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get("max");
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initAppLovin(activity);
                HlAdClient.initSuccessMap.put("max", true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mLovinBean.setStartTime(System.currentTimeMillis());
        if (parameter != null) {
            MaxNativeAdViewBinder.Builder builder = new MaxNativeAdViewBinder.Builder(parameter.getAdLayoutId());
            builder.setMediaContentViewGroupId(parameter.getAdContentId());
            builder.setTitleTextViewId(parameter.getAdTitleId());
            builder.setBodyTextViewId(parameter.getAdBodyTitleId());
            builder.setOptionsContentViewGroupId(parameter.getAdOptionsId());
            nativeAdView = new MaxNativeAdView(builder.build(), activity);
        }
        nativeAdLoader = new MaxNativeAdLoader(bean.adSlotId, activity);
        nativeAdLoader.setNativeAdListener(this);
        nativeAdLoader.setRevenueListener(maxAd -> {
            if (isShow) {
                isShow = false;
                mNativeListener.onADShow(mNativeView, InitDataProcessor.SDK_MAX, mLovinBean, maxAd.getNetworkName());
            }
        });
    }


    @Override
    public void loadAd() {
        if (nativeAdView != null) {
            nativeAdLoader.loadAd(nativeAdView);
        }
    }

    @Override
    public void onNativeAdLoaded(@Nullable final MaxNativeAdView nativeAdView, final MaxAd maxAd) {
        mNativeView = nativeAdView;
        mMaxAd = maxAd;
        mLovinBean.setEndTime(System.currentTimeMillis());
        double ecpm = maxAd.getRevenue()*1000;
        Log.e("11111", "====onUnifiedLoaded====" + maxAd.getNetworkName() + "===ecpm===" + ecpm);
        float ratio=0.0f;
//        if(nativeAdView!=null&&maxAd.getNativeAd()!=null){
//            try{
//                ratio=maxAd.getNativeAd().getMediaContentAspectRatio();
////                if(ratio>0){
////                    Log.e("11111", "====ratio==== " +ratio+"---"+ (int) (PxUtil.getDeviceWidthInPixel(mActivity) /ratio));
////                    Log.e("11111", "====ratio==== " + PxUtil.getDeviceWidthInPixel(mActivity));
////                    ViewGroup.LayoutParams layoutParams=nativeAdView.getLayoutParams();
////                    layoutParams.height= 200;
////                    nativeAdView.setLayoutParams(layoutParams);
////                }
//            }catch (Exception e){
//                e.printStackTrace();
//            }
//        }
        mLovinBean.setEp(ecpm);
        EcpmCompareBean bean = CompareEcpmUtils.compareEcpm(mLovinBean, ecpm);
        Log.e("11111", "===ecp1111m===" + bean.getEcpm());
        mLovinBean.setOp(bean.getEcpm());
        if (bean.getResult()) {
            mNativeListener.onADLoadView(nativeAdView, InitDataProcessor.SDK_MAX, mLovinBean, bean.getEcpm(), maxAd.getNetworkName(),ratio);
        } else {
            if (mNativeListener != null) {
                mNativeListener.onADError("max:竞价失败", 102, InitDataProcessor.SDK_MAX, mLovinBean);
            }
        }
    }

    @Override
    public void onNativeAdLoadFailed(String adUnitId, MaxError error) {
        mLovinBean.setEndTime(System.currentTimeMillis());
        mNativeListener.onADError(error.getMessage(), error.getCode(), InitDataProcessor.SDK_MAX, mLovinBean);
    }

    @Override
    public void onNativeAdClicked(final MaxAd ad) {
        if (!isClick) {
            isClick = true;
            mNativeListener.onADClick(mNativeView, mLovinBean, ad.getNetworkName());
        }
    }

    @Override
    public void onNativeAdExpired(final MaxAd nativeAd) {
    }

    @Override
    public void release() {
        if (mMaxAd != null) {
            nativeAdLoader.destroy(mMaxAd);
        }
        nativeAdLoader.destroy();
    }
}
