package com.halo.halojuhesdk.core.channel.hlapi;

import android.app.Activity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;

import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.channel.IBannerAd;
import com.halo.halojuhesdk.core.channel.IBannerProxyListener;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.PxUtil;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halomobi.sdk.HMBannerAd;
import com.halomobi.sdk.HlAdParameter;
import com.halomobi.sdk.listener.BannerListener;


public class HlApiBannerAd implements IBannerAd, BannerListener {
    private final Activity mContext;
    private final IBannerProxyListener mListener;
    private final SlotBean mApiBean;
    private HMBannerAd hmBannerAd;
    private boolean isReady = true;
    private boolean isFirstShow = true;
    private boolean isFirstClick = false;

    public HlApiBannerAd(Activity context, SlotBean hlApiBean, IBannerProxyListener listener) {
        mContext = context;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get(Config.mAppId);
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initApi(context);
                HlAdClient.initSuccessMap.put(Config.mAppId, true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mListener = listener;
        mApiBean = hlApiBean;
        mApiBean.setStartTime(System.currentTimeMillis());
        HlAdParameter adParameter = new HlAdParameter.Builder()
                .setSlotAdId(hlApiBean.slotId)
                .setAdWidth(PxUtil.getDeviceWidthInPixel(context))
                .setAdHeight(PxUtil.dpToPx(context, 50))
                .build();
        hmBannerAd = new HMBannerAd(mContext, adParameter, this);
        HlAdClient.containApiMap.put(mApiBean.slotId, true);
    }

    @Override
    public void loadAd() {
        hmBannerAd.loadAd();
    }

    @Override
    public void showAd(ViewGroup viewGroup) {
        hmBannerAd.showView(viewGroup);
    }

    @Override
    public void release() {
    }

    @Override
    public void onDisplayAd() {
        if (isFirstShow) {
            isFirstShow = false;
            mListener.onDisplayAd(mApiBean, "api");
        }
    }

    @Override
    public void onClickAd() {
        if (!isFirstClick) {
            isFirstClick = true;
            mListener.onClickAd(mApiBean, "api");
        }
    }

    @Override
    public void onCloseAd() {
        mListener.onCloseAd();
    }

    @Override
    public void onAdReady() {
        mApiBean.setEndTime(System.currentTimeMillis());
        if (isReady) {
            isReady = false;
            double ecpm = hmBannerAd.getEcpm();
            mApiBean.setOp(ecpm);
            mApiBean.setEp(ecpm);
            if (ecpm < mApiBean.minPrice) {
                mListener.onAdError("apiBanner:价格低" + mApiBean.minPrice, 102, InitDataProcessor.SDK_API, mApiBean);
            } else {
                mListener.onAdReady(mApiBean, InitDataProcessor.SDK_API, ecpm, "api");
            }
        }
    }

    @Override
    public void onAdError(String msg, int code) {
        mApiBean.setEndTime(System.currentTimeMillis());
        mListener.onAdError("api:" + msg, code, InitDataProcessor.SDK_API, mApiBean);
    }
}
