package com.halo.halojuhesdk.core.channel.hlapi;

import android.app.Activity;

import com.halo.halojuhesdk.core.channel.IInsertAd;
import com.halo.halojuhesdk.core.channel.IInsertProxyListener;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halomobi.sdk.HMInsertAd;
import com.halomobi.sdk.HlAdParameter;
import com.halomobi.sdk.listener.InsertListener;
import com.halomobi.utils.PxUtil;

public class HlApiInsertAd implements IInsertAd, InsertListener {
    private final SlotBean mApiBean;
    private final IInsertProxyListener mListener;
    private HMInsertAd insertAd;

    public HlApiInsertAd(Activity context, SlotBean apiBean, IInsertProxyListener listener) {
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get(Config.mAppId);
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initApi(context);
                HlAdClient.initSuccessMap.put(Config.mAppId, true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mApiBean = apiBean;
        mListener = listener;
        mApiBean.setStartTime(System.currentTimeMillis());
        HlAdParameter adParameter = new HlAdParameter.Builder()
                .setSlotAdId(apiBean.slotId)
//                .setAdWidth(PxUtil.dpToPx(context, 260))
//                .setAdHeight(PxUtil.dpToPx(context, 400))
                .build();
        insertAd = new HMInsertAd(context, adParameter, this);
        HlAdClient.containApiMap.put(apiBean.slotId, true);
    }

    @Override
    public void loadAd() {
        try {
            insertAd.loadAd();
        } catch (Exception e) {
            e.getStackTrace();
        }
    }

    @Override
    public void showAd() {
        try {
            insertAd.showAd();
        } catch (Exception e) {
            e.getStackTrace();
        }
    }

    @Override
    public void release() {
//        insertAd.clearAd();
    }

    @Override
    public void onDisplayAd() {
        mListener.onDisplayAd(mApiBean);
    }

    @Override
    public void onClickAd() {
        mListener.onClickAd(mApiBean);
    }

    @Override
    public void onCloseAd() {
        mListener.onCloseAd();
    }

    @Override
    public void onAdReady() {
        mApiBean.setEndTime(System.currentTimeMillis());
        double ecpm = insertAd.getEcpm();
        mApiBean.setOp(ecpm);
        mApiBean.setEp(ecpm);
        if (ecpm < mApiBean.minPrice) {
            mListener.onAdError("apiInsert:价格低" + mApiBean.minPrice, 102, InitDataProcessor.SDK_API, mApiBean);
        } else {
            mListener.onAdReady(mApiBean, InitDataProcessor.SDK_API, ecpm);
        }
    }

    @Override
    public void onAdError(String msg, int code) {
        mApiBean.setEndTime(System.currentTimeMillis());
        mListener.onAdError("api:" + msg, code, InitDataProcessor.SDK_API, mApiBean);
    }

}
