package com.halo.halojuhesdk.core.channel.hlapi;

import android.app.Activity;
import android.view.View;
import android.view.ViewGroup;
import android.widget.ImageView;
import android.widget.TextView;

import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.channel.INativeAd;
import com.halo.halojuhesdk.core.channel.INativeProxyListener;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halo.halojuhesdk.sdk.HlUnifiedAdParameter;
import com.halomobi.sdk.HMNativeAd;
import com.halomobi.sdk.HlAdParameter;
import com.halomobi.sdk.listener.NativeAdListener;
import com.halomobi.sdk.listener.NativeResponse;
import com.halomobi.view.imageloader.ImageLoader;

import java.util.List;

public class HlApiNativeAd implements INativeAd, NativeAdListener {

    private final SlotBean mApiBean;
    private final INativeProxyListener mListener;
    private final HMNativeAd nativeAd;
    private final Activity mContext;
    private final HlUnifiedAdParameter mParameter;
    private View mNativeView;
    private boolean isShow = true;
    private boolean isClick = false;

    public HlApiNativeAd(Activity mActivity, SlotBean apiBean, HlUnifiedAdParameter parameter, INativeProxyListener listener) {
        mContext = mActivity;
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get(Config.mAppId);
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initApi(mActivity);
                HlAdClient.initSuccessMap.put(Config.mAppId, true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mParameter = parameter;
        mApiBean = apiBean;
        mListener = listener;
        mApiBean.setStartTime(System.currentTimeMillis());
        HlAdParameter adParameter = new HlAdParameter.Builder()
                .setSlotAdId(apiBean.slotId)
                .build();
        nativeAd = new HMNativeAd(mActivity, adParameter, this);
        HlAdClient.containApiMap.put(apiBean.slotId, true);
    }

    public void loadAd() {
        nativeAd.loadAd();
    }

    @Override
    public void onAdReach(NativeResponse response, View view) {
        mNativeView = view;
        mApiBean.setEndTime(System.currentTimeMillis());
        if (response != null && view != null) {
            View insertView = insertView(response);
            double ecpm = response.getPrice();
            LogUtils.e("===apiPrice===" + ecpm);
            mApiBean.setOp(ecpm);
            mApiBean.setEp(ecpm);
            if (ecpm < mApiBean.minPrice && ecpm > 0) {
                mListener.onADError("apiSplash:价格低", 102, InitDataProcessor.SDK_API, mApiBean);
            } else {
                mListener.onADLoadView(insertView, InitDataProcessor.SDK_API, mApiBean, ecpm, "api",0f);
            }
        }
    }

    @Override
    public void onAdShow() {
        if (isShow) {
            isShow = false;
            mListener.onADShow(mNativeView, InitDataProcessor.SDK_API, mApiBean, "api");
        }
    }

    @Override
    public void onAdClick() {
        if (!isClick) {
            isClick = true;
            mListener.onADClick(mNativeView, mApiBean, "api");
        }
    }

    @Override
    public void onAdClose() {
        mListener.onADClose(mNativeView);
    }

    @Override
    public void onAdError(String msg, int code) {
        mApiBean.setEndTime(System.currentTimeMillis());
        mListener.onADError("api:" + msg, code, InitDataProcessor.SDK_API, mApiBean);
    }

    @Override
    public void release() {
    }

    private View insertView(NativeResponse response) {
        int adTitleId = mParameter.getAdTitleId();
        int adSubTitleId = mParameter.getAdBodyTitleId();
        int adContentId = mParameter.getAdContentId();
        View inflate = View.inflate(mContext, mParameter.getAdLayoutId(), null);
        if (adTitleId > 0) {
            TextView mTitleAd = inflate.findViewById(adTitleId);
            mTitleAd.setText(response.getTitle());
        }
        if (adSubTitleId > 0) {
            TextView mSubTitle = inflate.findViewById(adSubTitleId);
            mSubTitle.setText(response.getAdDescription());
        }
        ImageView imageView = new ImageView(mContext);
        imageView.setScaleType(ImageView.ScaleType.FIT_CENTER);
        List<String> imgList = response.getImgList();
        if (imgList != null && !imgList.isEmpty()) {
            ImageLoader.load(imageView, imgList.get(0));
        }
        if (adContentId > 0) {
            ViewGroup mAdContentId = inflate.findViewById(adContentId);
            mAdContentId.addView(imageView);
        }
        return inflate;
    }
}
