package com.halo.halojuhesdk.core.channel.hlapi;

import android.app.Activity;
import android.content.Context;
import android.view.View;
import android.view.ViewGroup;
import android.widget.FrameLayout;
import android.widget.ImageView;
import android.widget.TextView;

import androidx.annotation.NonNull;

import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.channel.INativeAd;
import com.halo.halojuhesdk.core.channel.INativeProxyListener;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeAd;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeProxyListener;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halo.halojuhesdk.sdk.HlUnifiedAdParameter;
import com.halo.halojuhesdk.sdk.listener.HlNativeAdCallBackListener;
import com.halo.halojuhesdk.sdk.listener.HlNativeResponseImpl;
import com.halomobi.sdk.HMUnifiedNativeAd;
import com.halomobi.sdk.HlAdParameter;
import com.halomobi.sdk.listener.NativeAdListener;
import com.halomobi.sdk.listener.NativeResponse;
import com.halomobi.sdk.listener.NativeUnifiedBackAdListener;
import com.halomobi.sdk.listener.NativeUnifiedListener;
import com.halomobi.view.imageloader.ImageLoader;

import java.util.List;

public class HlApiUnifiedNativeAd implements INativeAd, NativeAdListener {
    private boolean isShow = true;
    private boolean isClick = false;
    private final SlotBean mApiBean;
    private final INativeProxyListener mListener;
    private final HMUnifiedNativeAd nativeAd;
    private final Activity mContext;
    private final HlUnifiedAdParameter mParameter;
    private View mNativeView;

    public HlApiUnifiedNativeAd(Activity mActivity, SlotBean apiBean, HlUnifiedAdParameter parameter, INativeProxyListener listener) {
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get(Config.mAppId);
        if (isInitSuccess == null || !isInitSuccess) {
            try {
                InitDataProcessor.initApi(mActivity);
                HlAdClient.initSuccessMap.put(Config.mAppId, true);
            } catch (Exception e) {
                e.printStackTrace();
            }
        }
        mContext = mActivity;
        mApiBean = apiBean;
        mListener = listener;
        mParameter = parameter;
        mApiBean.setStartTime(System.currentTimeMillis());
        HlAdParameter adParameter = new HlAdParameter.Builder()
                .setSlotAdId(apiBean.slotId)
                .build();
        nativeAd = new HMUnifiedNativeAd(mActivity, adParameter, this);
        HlAdClient.containApiMap.put(apiBean.slotId, true);
    }

    public void loadAd(){
        nativeAd.loadAd();
    }

    @Override
    public void onAdReach(NativeResponse response, View view) {
        mApiBean.setEndTime(System.currentTimeMillis());
        if (response != null) {
            double ecpm = response.getPrice();
            LogUtils.e("====apiUnified===" + ecpm);
            mApiBean.setOp(ecpm);
            mApiBean.setEp(ecpm);
            if (ecpm < mApiBean.minPrice && ecpm > 0) {
                mListener.onADError("apiSplash:价格低", 102, InitDataProcessor.SDK_API, mApiBean);
            } else {
                mNativeView = nativeAd.showViewById(mParameter.getAdLayoutId(), mParameter.getAdContentId(), mParameter.getAdTitleId(), mParameter.getAdBodyTitleId());
                mListener.onADLoadView(mNativeView, InitDataProcessor.SDK_API, mApiBean, ecpm, "api",0f);
            }
        }
    }

    @Override
    public void onAdError(String msg, int code) {
        mApiBean.setEndTime(System.currentTimeMillis());
        mListener.onADError("api:" + msg, code, InitDataProcessor.SDK_API, mApiBean);
    }

    @Override
    public void onAdShow() {
        if (isShow && mNativeView != null) {
            isShow = false;
            mListener.onADShow(mNativeView, InitDataProcessor.SDK_API, mApiBean, "api");
        }
    }

    @Override
    public void onAdClick() {
        if (!isClick && mNativeView != null) {
            isClick = true;
            mListener.onADClick(mNativeView, mApiBean, "api");
        }
    }

    @Override
    public void onAdClose() {

    }

    @Override
    public void release() {
    }
}
