package com.halo.halojuhesdk.core.request;

import android.content.Context;
import android.content.MutableContextWrapper;
import android.content.res.AssetManager;
import android.text.TextUtils;
import android.util.Log;
import androidx.annotation.Nullable;

import com.applovin.sdk.AppLovinMediationProvider;
import com.applovin.sdk.AppLovinSdk;
import com.applovin.sdk.AppLovinSdkSettings;
import com.halo.halojuhesdk.core.base.net.AbstractNetProcessor;
import com.halo.halojuhesdk.core.base.net.AbstractNetTask;
import com.halo.halojuhesdk.core.base.net.NetworkManager;
import com.halo.halojuhesdk.core.base.utils.KeyUtil;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.SystemUtils;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.sdk.HlAdClient;
import com.halomobi.sdk.HaloAdSdk;
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.logging.Logger;

import javax.net.ssl.HttpsURLConnection;

/**
 * 请求数据的percessor
 */

public class InitDataProcessor extends AbstractNetProcessor {
    private static final String TAG = "InitDataProcessor";
    public static final String SDK_API = "api";
    public static final String SDK_MAX = "sdk_310max";
    private static String mAppId;
    public static String ip;
    private static String mApiAppId;
    private static boolean mMaxInit;
    private static Context mContext;
    private static double bidfloor;

    public InitDataProcessor(Context context,String appId) {
        HlStatisManager.getInstance().cConfigRequestid();
        mContext = context;
        mAppId = appId;
    }

    @Override
    protected AbstractNetTask createTask() {
        return new InitDataTask(mAppId, loadBaseInfo());
    }

    @Override
    public void finish(int code, String content) {
        if (code == HttpsURLConnection.HTTP_OK) {
            try {
                LogUtils.e("====finish===" + content);
                judgeCode(content);
            } catch (Exception e) {
                e.printStackTrace();
            }
            reportSensorsData(0, code, content);
        } else {
            reportSensorsData(1, code, content);
            addFailSp();
        }
    }

    private void judgeCode(String content) {
        try {
            JSONObject object = new JSONObject(content);
            int errorCode = object.optInt("status");
            if (errorCode == 0) {
                removeFailSp();
                parseData2(object);
                if (MyUtils.getSp() != null) {
                    MyUtils.getSp().edit().putString("result", object.toString()).apply();
                }
            } else {
                addFailSp();
            }
        } catch (Exception e) {
            LogUtils.e("fs_sdk 初始化未知异常异常:");
            e.printStackTrace();
            addFailSp();
        }
    }


    @Override
    public void onError(int code, String content) {
        //根据code 判断是否需要增加失败次数
        Log.e("===fs_sdk初始化失败=== ", "code:  " + code + " content: " + content);
        if (code != -1) {
            addFailSp();
        }
        reportSensorsData(1, code, content);
    }

    //失败增加到sp中.
    private void addFailSp() {
        InitStatus.CURRENT_STATUS = InitStatus.INIT_FAIL;
    }

    //情况sp.
    private void removeFailSp() {
        InitStatus.CURRENT_STATUS = InitStatus.INIT_SUCCESS;
    }

    public static void parseData2(JSONObject object) {
        JSONObject dataObj = object.optJSONObject("data");
        if (dataObj != null) {
            JSONObject slotObj = dataObj.optJSONObject("slot");
            JSONObject configObj = dataObj.optJSONObject("config");
            ip = dataObj.optString("ip");
            if (configObj != null) {
                int tmax = configObj.optInt("tmax");
                if (tmax > 0) {
                    Config.AD_TIME_OUT = tmax;
                }
                bidfloor = configObj.optDouble("bidfloor");
                try{
                    if (TextUtils.isEmpty(HlAdClient.mConfigInfo)) {
                        JSONObject object1 = new JSONObject();
                        object1.put("ip", ip);
                        object1.put("bidfloor", bidfloor);
                        HlAdClient.mConfigInfo = object1.toString();
                    } else {
                        JSONObject jsonObject = new JSONObject(HlAdClient.mConfigInfo);
                        jsonObject.put("ip", ip);
                        jsonObject.put("bidfloor", bidfloor);
                        HlAdClient.mConfigInfo = jsonObject.toString();
                    }
                } catch (Exception e) {
                    e.printStackTrace();
                }
            }
            try {
                if (slotObj != null) {
                    Iterator<String> keyIter = slotObj.keys();
                    String key;
                    JSONArray value;
                    while (keyIter.hasNext()) {
                        key = keyIter.next();
                        Object obj = slotObj.get(key);
                        if (obj != null && !obj.equals("null")) {
                            if (obj instanceof JSONArray) {
                                value = (JSONArray) obj;
                                parseValueData(key, value);
                            }
                        }
                    }
                }
            } catch (JSONException e) {
                e.printStackTrace();
            }
            initSlot();
        }
    }

    public static void parseData3(JSONObject object) {
        JSONObject dataObj = object.optJSONObject("data");
        if (dataObj != null) {
            JSONObject slotObj = dataObj.optJSONObject("slot");
            if (slotObj != null) {
                try {
                    Iterator<String> keyIter = slotObj.keys();
                    String key;
                    JSONArray value;
                    while (keyIter.hasNext()) {
                        key = keyIter.next();
                        Object obj = slotObj.get(key);
                        if (obj != null && !obj.equals("null")) {
                            if (obj instanceof JSONArray) {
                                value = (JSONArray) obj;
                                parseValueData(key, value);
                            }
                        }
                    }
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        }
    }

    private static String loadBaseInfo() {
        JSONObject object = new JSONObject();
        try {
            if (!TextUtils.isEmpty(HlAdClient.mConfigInfo)) {
                try {
                    JSONObject jsonObject = new JSONObject(HlAdClient.mConfigInfo);
                    String oaId = jsonObject.optString("oaId");
                    String imei = jsonObject.optString("did");
                    String androidId = jsonObject.optString("dpid");
                    String macId = jsonObject.optString("mac");
                    object.put("oaid", oaId);
                    object.put("did", imei);
                    object.put("dpid", androidId);
                    object.put("mac", macId);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
            object.put("carrier", SystemUtils.getOperatorsName());
            object.put("make", SystemUtils.getVendor());
            object.put("model", SystemUtils.getDeviceModel());
            object.put("os", 2);
            object.put("osv", SystemUtils.getOSVersion());
            object.put("w", SystemUtils.getDeviceWidth());
            object.put("h", SystemUtils.getDeviceHeight());
            object.put("store_ver", SystemUtils.getAppStoreVersion());
            object.put("boot_mark", SystemUtils.getBoot());
            object.put("update_mark", SystemUtils.getUpdate());
            object.put("connection_type", Math.max(0, NetworkManager.getNetworkType(MyUtils.getContext())));
            object.put("device_type", SystemUtils.getDeviceType());
            //        Log.e("11111","===object===" + object);
            return KeyUtil.encryptCurrentAES(SystemUtils.formatCurrentData(), object.toString());
        } catch (Exception e) {
            e.printStackTrace();
        }
        return "";
    }

    private static void initSlot() {
        if (mMaxInit) {
            initAppLovin(mContext);
        }
    }

    public static void parseValueData(String key, JSONArray array) {
        List<SlotBean> slotBeanList = new ArrayList<>();
        if (array != null && array.length() > 0) {
            for (int i = 0; i < array.length(); i++) {
                try {
                    SlotBean slotBean = new SlotBean();
                    JSONObject slotObject = array.getJSONObject(i);
                    String slotId = slotObject.optString("slotId");
                    String adSlotId = slotObject.optString("adSlotId");
                    String appId = slotObject.optString("adAppId");
                    String adFrom = slotObject.optString("adFrom");
                    int slotType = slotObject.optInt("slotType");
                    String extendStr = slotObject.optString("extendStr");
                    String version = slotObject.optString("version");
                    String appName = slotObject.optString("appName");
                    String packageName = slotObject.optString("package");
                    double minPrice = slotObject.optDouble("minPrice");
                    double d = slotObject.optDouble("d");
                    double p = slotObject.optDouble("p");
                    double e = slotObject.optDouble("e");
                    if (!adFrom.contains("sdk")) {
                        mApiAppId = Config.mAppId;
                    } else if (adFrom.contains(SDK_MAX)) {
                        mMaxInit = true;
                    }
                    slotBean.setSlotId(slotId);
                    slotBean.setAdSlotId(adSlotId);
                    slotBean.setAppId(appId);
                    slotBean.setAdFrom(adFrom);
                    slotBean.setSlotType(slotType);
                    slotBean.setExtendStr(extendStr);
                    slotBean.setVersion(version);
                    slotBean.setAppName(appName);
                    slotBean.setPackageName(packageName);
                    slotBean.setPriority(slotBeanList.size());
                    slotBean.setMinPrice(minPrice);
                    slotBean.setD(d);
                    slotBean.setP(p);
                    slotBean.setE(e);
                    slotBeanList.add(slotBean);
                } catch (JSONException e) {
                    e.printStackTrace();
                }
            }
        }
        HlAdClient.slotMap.put(key, slotBeanList);
    }


    public static void initAppLovin(Context context) {
        Boolean isInitSuccess = HlAdClient.initSuccessMap.get("max");
        if (isInitSuccess == null || !isInitSuccess) {
            AppLovinSdkSettings sdkSettings = new AppLovinSdkSettings(context);
            sdkSettings.setLocationCollectionEnabled(false);
            AppLovinSdk instance = AppLovinSdk.getInstance(context);
            instance.getSettings().setMuted(true);
            instance.setMediationProvider(AppLovinMediationProvider.MAX);
            instance.initializeSdk( config -> {
                Log.e("11111","====initializeSdk====");
                HlAdClient.initSuccessMap.put("max", true);
            } );
        }
    }

    public static void initApi(Context context) {
        HaloAdSdk.init(context, Config.mAppId, HlAdClient.mConfigInfo);
    }

    private void reportSensorsData(int type, int code, String content) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("errorCode", code);
            jsonObject.put("msgContent", content);
            String eventIdStr = "";
            if (type == 0) {
                eventIdStr = "sdk_juhe_config_success";
            } else if (type == 1){
                eventIdStr = "sdk_juhe_config_error";
            }
            LogUtils.e("===reportConfigSensor===" + jsonObject);
            SensorsDataAPI.sharedInstance().track(eventIdStr, jsonObject);
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }
}
