package com.halo.halojuhesdk.core.request;

import android.text.TextUtils;
import android.util.Log;

import com.halo.halojuhesdk.core.base.LaunchMode;
import com.halo.halojuhesdk.core.base.ReqType;
import com.halo.halojuhesdk.core.base.net.AbstractNetTask;
import com.halo.halojuhesdk.core.common.utils.HttpUrlSettings;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.sdk.HlAdClient;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import java.io.OutputStream;
import java.net.HttpURLConnection;

/**
 * 统一请求类，如果需要post请求，请继承该类复写getEntity方法,和getReqType
 */

public class StatTask extends AbstractNetTask {
    private String mReqId;
    private String mPostjson;

    public StatTask(String postjson, String reqId) {
        super(MyUtils.getContext(), false);
        mPostjson = postjson;
        mReqId = reqId;
    }

    @Override
    public String getURL() {
        if (isContainApiAd()) {
            Log.e("11111","====getReportApiStartUrl====" + HttpUrlSettings.getReportApiStartUrl(mReqId));
            return HttpUrlSettings.getReportApiStartUrl(mReqId);
        }
        Log.e("11111","====getReportStartUrl====" + mReqId);
        return HttpUrlSettings.getReportStartUrl(mReqId);
    }

    private boolean isContainApiAd() {
        try {
//            Log.e("11111","===containApiMap.size====" + HlAdClient.containApiMap.size());
            JSONArray jsonArray = new JSONArray(mPostjson);
            if (jsonArray != null && jsonArray.length() > 0) {
                JSONObject object = jsonArray.getJSONObject(0);
                String slotId = object.optString("slotId");
                Boolean isContain = HlAdClient.containApiMap.get(slotId);
                if (isContain != null && isContain) {
                    return true;
                }
            }
        } catch (JSONException e) {
            e.printStackTrace();
        }
        return false;
    }

    /**
     * post请求
     *
     * @param outputStream
     */
    @Override
    protected void getEntity(OutputStream outputStream) {
        try {
            if (outputStream != null && !TextUtils.isEmpty(mPostjson)) {

                outputStream.write(mPostjson.getBytes());
            }
        } catch (Exception e) {
            e.printStackTrace();
        }

    }

    @Override
    protected void onAddHeaders(HttpURLConnection connection) {
        connection.setRequestProperty("Content-Type", "application/json");
    }

    @Override
    protected ReqType getReqType() {
        return ReqType.Post;
    }

    @Override
    public LaunchMode getLaunchMode() {
        return LaunchMode.addnew;
    }

    @Override
    public String getName() {
        return "StatTask";
    }
}
