package com.halo.halojuhesdk.core.request;


import com.halo.halojuhesdk.core.base.net.AbstractNetProcessor;
import com.halo.halojuhesdk.core.base.net.AbstractNetTask;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.sensorsdata.analytics.android.sdk.SensorsDataAPI;

import org.json.JSONArray;
import org.json.JSONException;
import org.json.JSONObject;

import javax.net.ssl.HttpsURLConnection;

/**
 * 请求数据的percessor
 */

public class StatisProcessor extends AbstractNetProcessor {
    private static final String TAG = "StatisProcessor";
    private String mErrorMsg;
    private SlotBean mBean;
    private String mStepName;
    private String mAction;
    private String mReqId;
    private RequestCallBack mCallBack;

    public StatisProcessor(SlotBean bean, String stepName, String action, String reqId, String errorMsg) {
        mBean = bean;
        mStepName = stepName;
        mAction = action;
        mReqId = reqId;
        mErrorMsg = errorMsg;
    }

    @Override
    protected AbstractNetTask createTask() {
        return new StatisTask(mBean, mStepName, mAction, mReqId, mErrorMsg);
    }


    @Override
    public void finish(int code, String content) {
        if (code == HttpsURLConnection.HTTP_OK) {
            if (mCallBack != null) {
                mCallBack.onSuccess();
            }
        }
        LogUtils.e("===reportStatis==" + mReqId + "===soltId===" + mBean.slotId);
        reportSensorsData(0, code, content);
    }


    @Override
    public void onError(int code, String content) {
        if (mCallBack != null) {
            mCallBack.onError();
        }
        reportSensorsData(1, code, content);
    }

    public void callBack(RequestCallBack requestCallBack) {
        mCallBack = requestCallBack;
    }

    private void reportSensorsData(int type, int code, String content) {
        JSONObject jsonObject = new JSONObject();
        try {
            jsonObject.put("errorCode", code);
            jsonObject.put("msgContent", content);
            jsonObject.put("slotId", mBean.slotId);
            jsonObject.put("action", mAction);
            jsonObject.put("stepName", mStepName);
            String eventIdStr = "";
            if (type == 0) {
                eventIdStr = "sdk_juhe_report_success";
            } else if (type == 1){
                eventIdStr = "sdk_juhe_report_error";
            }
            LogUtils.e("===reportActionSensor===" + jsonObject);
            SensorsDataAPI.sharedInstance().track(eventIdStr, jsonObject);
        } catch (JSONException e) {
            throw new RuntimeException(e);
        }
    }
}
