package com.halo.halojuhesdk.sdk;

import android.app.Activity;
import android.text.TextUtils;
import android.view.View;
import android.view.ViewGroup;

import com.halo.halojuhesdk.core.base.utils.AnalyseSlotBeanUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.Utils;
import com.halo.halojuhesdk.core.channel.IBannerAd;
import com.halo.halojuhesdk.core.channel.IBannerProxyListener;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinBanner;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinInsert;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiBannerAd;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiInsertAd;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.InitStatus;
import com.halo.halojuhesdk.core.request.RequestAdManager;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.core.request.UpRequestCallBack;
import com.halo.halojuhesdk.sdk.listener.HlBannerListener;
import com.halomobi.sdk.HMBannerAd;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

public class HlBannerAd {
    private String mSlotId;
    private Activity mActivity;
    private HlBannerListener mAdListener;
    private Map<String, Boolean> bannerAdMap;
    private Map<String, IBannerAd> bannerMap;
    private Map<String, SlotBean> bannerPriceMap;
    private Map<String, Double> ecpmAdMap;
    private Map<String, String> adSubTypeMap;
    private int maxTime;
    private boolean isLoadAd;
    private JSONArray mPostJson;
    double currentPrice = 0;
    private Timer mTimer;
    private TimerTask mTask;
    private String mBannerReqId;
    private String showAdSlotId;

    public HlBannerAd(Activity context, String slotId, HlBannerListener listener) {
        MyUtils.init(context);
        loadBanner(context, slotId, listener);
    }

    public void setRefresh(int refreshCount) {
        if (refreshCount >= 30 && refreshCount <= 120) {
            mTimer = new Timer();
            mTask = new TimerTask() {
                @Override
                public void run() {
                    mActivity.runOnUiThread(() -> {
                        if (mAdListener != null) {
                            loadBanner(mActivity, mSlotId, mAdListener);
                            loadAd();
                        }
                    });
                }
            };
            mTimer.schedule(mTask, refreshCount*1000, refreshCount*1000);
        }
    }

    private void loadBanner(Activity context, String slotId, HlBannerListener listener) {
        mBannerReqId = UUID.randomUUID().toString();
        bannerAdMap = new HashMap<>();
        bannerMap = new HashMap<>();
        bannerPriceMap = new HashMap<>();
        ecpmAdMap = new HashMap<>();
        adSubTypeMap = new HashMap<>();
        mPostJson = new JSONArray();
        maxTime = Config.AD_TIME_OUT;
        mSlotId = slotId;
        isLoadAd = false;
        mAdListener = listener;
        this.mActivity = context;
        List<SlotBean> slotBeanList = RequestAdManager.requestChannel(slotId);
        if (slotBeanList == null || slotBeanList.size() == 0) {
            if (mAdListener != null) {
                mAdListener.onAdError("横幅广告初始化失败", InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("横幅广告加载失败:", InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        } else {
            try {
                initBanner(context, slotBeanList);
            } catch (Exception e) {
                e.printStackTrace();
                mAdListener.onAdError("横幅广告加载失败:" + e, InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("横幅广告加载失败:" + e, InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        }
    }

    private void initBanner(Activity context, final List<SlotBean> baseBean) {
        HlStatisManager.getInstance().cBannerRequestid();
        final List<SlotBean> currentList = new ArrayList<>();
        for (int i = 0; i < baseBean.size(); i++) {
            SlotBean bean = baseBean.get(i);
            if (bean == null) {
                continue;
            }
            String adFrom = bean.getAdFrom();
            if (adFrom.equals(InitDataProcessor.SDK_MAX) && HlAdClient.mTypeList.contains(Config.JuHeAdType.MAX)) {
                AppLovinBanner appLovinInsert = new AppLovinBanner(context, bean, proxyListener);
                currentList.add(bean);
                bannerMap.put(bean.adSlotId, appLovinInsert);
            }
            else if (adFrom.equals(InitDataProcessor.SDK_API) && HlAdClient.mTypeList.contains(Config.JuHeAdType.API)) {
                HlApiBannerAd apiBanner = new HlApiBannerAd(context, bean, proxyListener);
                currentList.add(bean);
                bannerMap.put(bean.adSlotId, apiBanner);
            }
        }
        if (currentList.isEmpty()) {
            mAdListener.onAdError("横幅广告初始化失败, 无相应广告配置", InitStatus.INIT_FAIL_CODE);
            reportErrorMsg("横幅广告初始化失败, 无相应广告配置", InitStatus.INIT_FAIL_CODE, "广告源", null);
            return;
        }
        final Timer timer = new Timer();
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                maxTime = maxTime - 100;
                if (bannerAdMap.size() == currentList.size()) {
                    timer.cancel();
                    formatBannerResult(currentList);
                } else if (maxTime <= 0) {
                    timer.cancel();
                    if (!bannerAdMap.isEmpty()) {
                        if (isLoadAd) return;
                        formatBannerResult(currentList);
                    } else {
                        for (SlotBean bean : currentList) {
                            analyseBeanObj(bean, false,false, 0);
                        }
                        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mBannerReqId);
                        onAdError("超时没有加载到横幅广告");
                    }
                }
            }
        };
        timer.schedule(timerTask, 0, 100);
    }

    private void formatBannerResult(List<SlotBean> currentList) {
        String currentAdSlotId = "";
        for (Map.Entry<String, Double> entry : ecpmAdMap.entrySet()) {
            String key = entry.getKey();
            double value = entry.getValue();
            if (value >= currentPrice) {
                currentAdSlotId = key;
                currentPrice = value;
            }
        }
        if ((currentPrice == 0 || currentPrice == 1) && !currentList.isEmpty()) {
            for (SlotBean bean : currentList) {
                Boolean aBoolean = bannerAdMap.get(bean.adSlotId);
                if (aBoolean != null && aBoolean) {
                    currentAdSlotId = bean.adSlotId;
                    break;
                }
            }
        }
//        LogUtils.e("=====bannersize====" + ecpmAdMap.size() + "===currentAdSlotId===" + currentAdSlotId + "====currentPrice====" + currentPrice);
        for (SlotBean bean : currentList) {
            final String adFrom = bean.adFrom;
            String adSlotId = bean.adSlotId;
            Boolean aBoolean = bannerAdMap.get(adSlotId);
            if (aBoolean != null && aBoolean) {
                Double ecpmNum = ecpmAdMap.get(adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                bean.setOp(ecpmNum);
                if (!isLoadAd && !TextUtils.isEmpty(currentAdSlotId) && currentAdSlotId.equals(adSlotId)) {
                    if (bannerPriceMap != null && bannerPriceMap.containsKey(adSlotId)) {
                        analyseBeanObj(bean, true, false, ecpmNum);
                        continue;
                    }
                    isLoadAd = true;
                    showAdSlotId = adSlotId;
                    mActivity.runOnUiThread(() -> {
                        String adType = adSubTypeMap.get(adSlotId);
                        mAdListener.onAdReady(adType);
                        LogUtils.e("====adFrom111====" + adFrom);
                    });
                    analyseBeanObj(bean, true, true, ecpmNum);
                } else {
                    analyseBeanObj(bean, true, false, ecpmNum);
                }
            } else{
                analyseBeanObj(bean, false, false, 0);
            }
        }
        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mBannerReqId);
        if (!isLoadAd) {
            onAdError("没有加载到横幅广告源");
        }
    }

    private void analyseBeanObj(SlotBean mBean, boolean isQuota, boolean isUse, double op) {
        try {
            JSONObject parameJson = AnalyseSlotBeanUtils.analyseBean(mBean, isQuota, isUse, op, mBannerReqId);
            mPostJson.put(parameJson);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void onAdError(final String errorMsg) {
        reportErrorMsg(errorMsg, 100, "广告源", null);
        mActivity.runOnUiThread(() -> {
            LogUtils.e("====error====" + errorMsg);
            mAdListener.onAdError(errorMsg, 100);
        });
    }

    /**
     * 请求广告是否成功
     */
    public void loadAd() {
        if (bannerMap != null) {
            for (Map.Entry<String, IBannerAd> entry : bannerMap.entrySet()) {
                entry.getValue().loadAd();
            }
        } else {
            mActivity.runOnUiThread(() -> mAdListener.onAdError("横幅广告load失败:", InitStatus.INIT_FAIL_CODE));
            reportErrorMsg("横幅广告load失败:", InitStatus.INIT_FAIL_CODE, "广告源", null);
        }
    }

    public void showView(ViewGroup viewGroup) {
        if (bannerMap != null && bannerMap.containsKey(showAdSlotId)) {
            IBannerAd iBannerAd = bannerMap.get(showAdSlotId);
            if (iBannerAd != null) {
                iBannerAd.showAd(viewGroup);
            }
        }
    }

    public double getECPM() {
        return currentPrice;
    }

    private IBannerProxyListener proxyListener = new IBannerProxyListener() {
        @Override
        public void onDisplayAd(SlotBean bean, String adType) {
            if (mAdListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_IMP, mBannerReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mAdListener.onDisplayAd(adType);
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mAdListener.onDisplayAd(adType);
                }
            }
        }

        @Override
        public void onClickAd(SlotBean bean, String adType) {
            if (mAdListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_CLICK, mBannerReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mAdListener.onClickAd(adType);
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mAdListener.onClickAd(adType);
                }
            }
        }

        @Override
        public void onCloseAd() {
            if (mAdListener != null) {
                mAdListener.onCloseAd();
            }
            release();
        }

        @Override
        public void onAdReady(SlotBean bean, String adType, double ecpm, String adSubType) {
            LogUtils.e("====onAdReady===" + adType + "====adSlotId===" + bean.adSlotId);
            bannerAdMap.put(bean.adSlotId, true);
            ecpmAdMap.put(bean.adSlotId, ecpm);
            adSubTypeMap.put(bean.adSlotId, adSubType);
        }

        @Override
        public void onAdError(String msg, int code, String adType, SlotBean bean) {
            LogUtils.e("====error===" + msg + "===code===" + code + "===adType===" + adType + "====adSlotId===" + bean.adSlotId);
            if (code == 102) {
                bannerPriceMap.put(bean.adSlotId, bean);
                bannerAdMap.put(bean.adSlotId, true);
            } else {
                bannerAdMap.put(bean.adSlotId, false);
            }
            reportErrorMsg(msg, code, adType, bean);
        }
    };

    private void reportErrorMsg(String msg, int code, String type, SlotBean bean) {
        if (bean == null) {
            bean = new SlotBean();
            bean.setAppId(Config.mAppId);
            bean.setSlotId(mSlotId);
        }
        String errorMsg = type + ": errorTime==" + Utils.formatCurrentTime() + "==errorMsg:" + msg + "==errorCode==" + code;
        HlStatisManager.getInstance().reportStaticError(bean, HlStatisManager.STEPNAME_ERROR, "", mBannerReqId, errorMsg);
    }

    public void release() {
        destroyView();
        destroyTimer();
    }

    private void destroyTimer() {
        if (mTask != null) {
            mTask.cancel();
            mTask = null;
        }
        if (mTimer != null) {
            mTimer.cancel();
            mTimer = null;
        }
    }

    private void destroyView() {
        if (bannerMap != null) {
            for (Map.Entry<String, IBannerAd> entry : bannerMap.entrySet()) {
                entry.getValue().release();
            }
        }
    }
}
