package com.halo.halojuhesdk.sdk;

import android.app.Activity;
import android.text.TextUtils;

import com.halo.halojuhesdk.core.base.utils.AnalyseSlotBeanUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.Utils;
import com.halo.halojuhesdk.core.channel.IInsertAd;
import com.halo.halojuhesdk.core.channel.IInsertProxyListener;
import com.halo.halojuhesdk.core.channel.IInsertVideoProxyListener;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinInsert;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiInsertAd;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.InitStatus;
import com.halo.halojuhesdk.core.request.RequestAdManager;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.core.request.UpRequestCallBack;
import com.halo.halojuhesdk.sdk.listener.HlInsertAdListener;
import com.halo.halojuhesdk.sdk.listener.HlInsertVideoListener;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

/**
 * @author Feng Zhanao
 * @date 2018/9/27
 * @description
 */
public class HlInsertAd {
    private String mSlotId;
    private Activity mActivity;
    private HlInsertAdListener mAdListener;
    private Map<String, Boolean> insertAdMap;
    private Map<String, SlotBean> insertPriceAdMap;
    private Map<String, Double> ecpmAdMap;
    private int maxTime;
    private boolean isLoadAd;
    private JSONArray mPostJson;
    private HlInsertVideoListener mVideoListener;
    private Map<String, IInsertAd> insertViewMap;
    private String mShowAdSlotId;
    double currentPrice = 0;
    private String mInsertReqId = UUID.randomUUID().toString();
    public HlInsertAd(Activity context, String slotId, HlInsertAdListener adListener) {
        MyUtils.init(context);
        mSlotId = slotId;
        mPostJson = new JSONArray();
        insertAdMap = new HashMap<>();
        insertPriceAdMap = new HashMap<>();
        ecpmAdMap = new HashMap<>();
        maxTime = Config.AD_TIME_OUT;
        isLoadAd = false;
        this.mAdListener = adListener;
        mActivity = context;
        insertViewMap = new HashMap<>();
        List<SlotBean> slotBeanList = RequestAdManager.requestChannel(slotId);
        if (slotBeanList == null || slotBeanList.size() == 0) {
            mAdListener.onAdError("插屏广告初始化失败", InitStatus.REQUEST_FAIL_CODE);
            reportErrorMsg("插屏广告加载失败:", InitStatus.REQUEST_FAIL_CODE, "广告源", null);
        } else {
            try {
                initInsert(context, slotBeanList);
            } catch (Exception e) {
                e.printStackTrace();
                mAdListener.onAdError("插屏广告加载失败:" + e, InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("插屏广告加载失败:" + e, InitStatus.REQUEST_FAIL_CODE, "广告源", null);
            }
        }
    }

    private void initInsert(Activity context, final List<SlotBean> baseBean) {
        HlStatisManager.getInstance().cInsertRequestid();
        final List<SlotBean> currentList = new ArrayList<>();
        for (int i = 0; i < baseBean.size(); i++) {
            SlotBean bean = baseBean.get(i);
            if (bean == null) {
                continue;
            }
            String adFrom = bean.getAdFrom();
            if (adFrom.equals(InitDataProcessor.SDK_MAX) && HlAdClient.mTypeList.contains(Config.JuHeAdType.MAX)) {
                AppLovinInsert appLovinInsert = new AppLovinInsert(context, bean, proxyListener);
                currentList.add(bean);
                insertViewMap.put(bean.adSlotId, appLovinInsert);
            } else if (adFrom.equals(InitDataProcessor.SDK_API) && HlAdClient.mTypeList.contains(Config.JuHeAdType.API)) {
                HlApiInsertAd apiInsert = new HlApiInsertAd(context, bean, proxyListener);
                currentList.add(bean);
                insertViewMap.put(bean.adSlotId, apiInsert);
            }
        }
        if (currentList.isEmpty()) {
            mAdListener.onAdError("插屏广告初始化失败, 无相应广告配置", InitStatus.REQUEST_FAIL_CODE);
            reportErrorMsg("插屏广告初始化失败, 无相应广告配置", InitStatus.REQUEST_FAIL_CODE, "广告源", null);
            return;
        }
        final Timer timer = new Timer();
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                maxTime = maxTime - 100;
                if (insertAdMap.size() == currentList.size()) {
                    timer.cancel();
                    formatInsertResult(currentList);
                } else if (maxTime <= 0) {
                    timer.cancel();
                    if (insertAdMap.size() > 0) {
                        if (isLoadAd) return;
                        formatInsertResult(currentList);
                    } else {
                        for (SlotBean bean : currentList) {
                            analyseBeanObj(bean, false, false, 0);
                        }
                        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mInsertReqId);
                        onAdError("超时没有加载到插屏广告");
                    }
                }
            }
        };
        timer.schedule(timerTask, 0, 100);
    }

    private void formatInsertResult(List<SlotBean> currentList) {
        String currentAdSlotId = "";
        for (Map.Entry<String, Double> entry : ecpmAdMap.entrySet()) {
            String key = entry.getKey();
            double value = entry.getValue();
            if (value >= currentPrice) {
                currentAdSlotId = key;
                currentPrice = value;
            }
        }
        if ((currentPrice == 0 || currentPrice == 1) && !currentList.isEmpty()) {
            for (SlotBean bean : currentList) {
                Boolean aBoolean = insertAdMap.get(bean.adSlotId);
                if (aBoolean != null && aBoolean) {
                    currentAdSlotId = bean.adSlotId;
                    break;
                }
            }
        }
//        LogUtils.e("=====insertsize====" + ecpmAdMap.size() + "===currentAdSlotId===" + currentAdSlotId + "====currentPrice====" + currentPrice);
        for (SlotBean bean : currentList) {
            final String adFrom = bean.adFrom;
            final String adSlotId = bean.adSlotId;
            Boolean aBoolean = insertAdMap.get(adSlotId);
            if (aBoolean != null && aBoolean) {
                Double ecpmNum = ecpmAdMap.get(adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                bean.setOp(ecpmNum);
                if (!isLoadAd && !TextUtils.isEmpty(currentAdSlotId) && currentAdSlotId.equals(adSlotId)) {
                    if (insertPriceAdMap != null && insertPriceAdMap.containsKey(adSlotId)) {
                        analyseBeanObj(bean, true, false, ecpmNum);
                        continue;
                    }
                    isLoadAd = true;
                    mActivity.runOnUiThread(new Runnable() {
                        @Override
                        public void run() {
                            mShowAdSlotId = adSlotId;
                            mAdListener.onAdReady();
                            LogUtils.e("====adFrom111====" + adFrom);
                        }
                    });
                    analyseBeanObj(bean, true, true, ecpmNum);
                } else {
                    analyseBeanObj(bean, true, false, ecpmNum);
                }
            } else{
                analyseBeanObj(bean, false, false, 0);
            }
        }
        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mInsertReqId);
        if (!isLoadAd) {
            onAdError( "没有加载到插屏广告源");
        }
    }

    private void analyseBeanObj(SlotBean mBean, boolean isQuota, boolean isUse, double op) {
        try {
            JSONObject parameJson = AnalyseSlotBeanUtils.analyseBean(mBean, isQuota, isUse, op, mInsertReqId);
            mPostJson.put(parameJson);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void onAdError(final String errorMsg) {
        mActivity.runOnUiThread(new Runnable() {
            @Override
            public void run() {
                LogUtils.e("====error====" + errorMsg);
                mAdListener.onAdError(errorMsg, 100);
            }
        });
        reportErrorMsg(errorMsg, 100, "广告源", null);
    }

    public void loadAd() {
        if (insertViewMap != null) {
            for (Map.Entry<String, IInsertAd> entry : insertViewMap.entrySet()) {
                entry.getValue().loadAd();
            }
        } else {
            mActivity.runOnUiThread(new Runnable() {
                @Override
                public void run() {
                    mAdListener.onAdError("插屏广告load失败:", InitStatus.INIT_FAIL_CODE);
                }
            });
            reportErrorMsg("插屏广告load失败:", InitStatus.INIT_FAIL_CODE, "广告源", null);
        }
    }

    public double getECPM() {
        return currentPrice;
    }

    public Map<String, Double> getEcpmMap() {
        return ecpmAdMap;
    }

    public void show() {
        if (insertViewMap != null && !TextUtils.isEmpty(mShowAdSlotId)) {
            IInsertAd iInsertAd = insertViewMap.get(mShowAdSlotId);
            if (iInsertAd != null) {
                iInsertAd.showAd();
            }
        }
    }

    private IInsertProxyListener proxyListener = new IInsertProxyListener() {
        @Override
        public void onDisplayAd(SlotBean bean) {
            if (mAdListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_IMP, mInsertReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mAdListener.onDisplayAd();
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mAdListener.onDisplayAd();
                }
            }
        }

        @Override
        public void onClickAd(SlotBean bean) {
            if (mAdListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_CLICK, mInsertReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mAdListener.onClickAd();
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mAdListener.onClickAd();
                }
            }
        }

        @Override
        public void onCloseAd() {
            if (mAdListener != null) {
                mAdListener.onCloseAd();
            }
        }

        @Override
        public void onAdReady(SlotBean bean, String adType, double ecpm) {
            LogUtils.e("====onAdReady===" + adType + "===bean===" + bean.adSlotId + "===ecmp===" + ecpm);
            insertAdMap.put(bean.adSlotId, true);
            ecpmAdMap.put(bean.adSlotId, ecpm);
        }

        @Override
        public void onAdError(String msg, int code, String adType, SlotBean bean) {
            LogUtils.e("====error===" + msg + "===code===" + code + "===adType===" + adType);
            if (code == 102) {
                insertPriceAdMap.put(bean.adSlotId, bean);
                insertAdMap.put(bean.adSlotId, true);
            } else {
                insertAdMap.put(bean.adSlotId, false);
            }
            reportErrorMsg(msg, code, adType, bean);
        }
    };

    private void reportErrorMsg(String msg, int code, String type, SlotBean bean) {
        if (bean == null) {
            bean = new SlotBean();
            bean.setAppId(Config.mAppId);
            bean.setSlotId(mSlotId);
        }
        String errorMsg = type + ": errorTime==" + Utils.formatCurrentTime() + "==errorMsg:" + msg + "==errorCode==" + code;
        HlStatisManager.getInstance().reportStaticError(bean, HlStatisManager.STEPNAME_ERROR, "", mInsertReqId, errorMsg);
    }

    private IInsertVideoProxyListener videoProxyListener = new IInsertVideoProxyListener() {

        @Override
        public void onVideoReady(long videoDuration) {
            if (mVideoListener != null) {
                mVideoListener.onVideoReady(videoDuration);
            }
        }

        @Override
        public void onVideoStart(SlotBean bean) {
            if (mVideoListener != null) {
                mVideoListener.onVideoStart();
            }
        }

        @Override
        public void onVideoPause() {
            if (mVideoListener != null) {
                mVideoListener.onVideoPause();
            }
        }

        @Override
        public void onVideoComplete(SlotBean bean) {
            if (mVideoListener != null) {
                mVideoListener.onVideoComplete();
            }
        }

        @Override
        public void onVideoError(int errorCode, String errorMsg) {
            if (mVideoListener != null) {
                mVideoListener.onVideoError(errorCode, errorMsg);
            }
        }

        @Override
        public void onVideoPageOpen() {
            if (mVideoListener != null) {
                mVideoListener.onVideoPageOpen();
            }
        }

        @Override
        public void onVideoPageClose() {
            if (mVideoListener != null) {
                mVideoListener.onVideoPageClose();
            }
        }

        @Override
        public void onSkippedVideo() {
            if (mVideoListener != null) {
                mVideoListener.onSkippedVideo();
            }
        }
    };

    public void destroy() {

    }
}

