package com.halo.halojuhesdk.sdk;

import android.app.Activity;
import android.text.TextUtils;
import android.view.View;

import com.halo.halojuhesdk.core.base.utils.AnalyseSlotBeanUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.Utils;
import com.halo.halojuhesdk.core.channel.INativeAd;
import com.halo.halojuhesdk.core.channel.INativeProxyListener;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeAd;
import com.halo.halojuhesdk.core.channel.IUnifiedNativeProxyListener;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinNative;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinUnifiedNative;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiNativeAd;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiUnifiedNativeAd;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.InitStatus;
import com.halo.halojuhesdk.core.request.RequestAdManager;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.core.request.UpRequestCallBack;
import com.halo.halojuhesdk.sdk.listener.HlNativeAdListener;
import com.halo.halojuhesdk.sdk.listener.HlNativeResponse;
import com.halo.halojuhesdk.sdk.listener.HlUnifiedNativeAdListener;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

public class HlNativeUnifiedAd {
    private final HlUnifiedAdParameter mParameter;
    private String mSlotId;
    private Activity mActivity;
    private HlNativeAdListener mHlNativeAdListener;
    private Map<String, Boolean> nativeSelectMap;
    private Map<String, INativeAd> nativeAdMap;
    private Map<String, View> nativeViewAdMap;
    private Map<String, SlotBean> nativePriceAdMap;
    private HashMap<String, Double> ecpmAdMap;
    private Map<String, String> adSubTypeMap;
    private Map<String, Float> adRatioMap;
    private int maxTime;
    private boolean isLoadAd;
    private String mAdFrom;
    private JSONArray mPostJson;
    double currentPrice = 0;
    private boolean isAdClose;
    private String mNativeUnifiedReqId = UUID.randomUUID().toString();

    public HlNativeUnifiedAd(Activity context, String slotId, HlUnifiedAdParameter parameter, HlNativeAdListener nativeAdListener) {
        MyUtils.init(context);
        mParameter = parameter;
        nativeViewAdMap = new HashMap<>();
        mPostJson = new JSONArray();
        maxTime = Config.AD_TIME_OUT;
        isLoadAd = false;
        mSlotId = slotId;
        nativeSelectMap = new HashMap<>();
        nativeAdMap = new HashMap<>();
        nativePriceAdMap = new HashMap<>();
        ecpmAdMap = new HashMap<>();
        adSubTypeMap = new HashMap<>();
        adRatioMap = new HashMap<>();
        mHlNativeAdListener = nativeAdListener;
        mActivity = context;
        List<SlotBean> slotBeanList = RequestAdManager.requestChannel(slotId);
        if (slotBeanList == null || slotBeanList.isEmpty()) {
            mHlNativeAdListener.onADError("未加载信息流自渲染广告", InitStatus.INIT_FAIL_CODE);
            reportErrorMsg("未加载信息流自渲染广告", InitStatus.INIT_FAIL_CODE, "初始化", null);
        } else {
            try {
                initNative(slotBeanList);
            } catch (Exception e) {
                e.printStackTrace();
                mHlNativeAdListener.onADError("未加载信息流自渲染广告:" + e, InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("未加载信息流广告:" + e, InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        }
    }

    private void initNative(final List<SlotBean> baseBean) {
        HlStatisManager.getInstance().cNativeRequestid();
        final List<SlotBean> currentList = new ArrayList<>();
        for (int i = 0; i < baseBean.size(); i++) {
            SlotBean bean = baseBean.get(i);
            String adFrom = bean.getAdFrom();
            bean.setReqId(mNativeUnifiedReqId);
            if (adFrom.equals(InitDataProcessor.SDK_MAX) && HlAdClient.mTypeList.contains(Config.JuHeAdType.MAX)) {
                AppLovinUnifiedNative appLovinNative = new AppLovinUnifiedNative(mActivity, bean, mParameter, proxyListener);
                currentList.add(bean);
                nativeAdMap.put(bean.adSlotId, appLovinNative);
            } else
                if (adFrom.equals(InitDataProcessor.SDK_API) && HlAdClient.mTypeList.contains(Config.JuHeAdType.API)) {
                HlApiUnifiedNativeAd apiNative = new HlApiUnifiedNativeAd(mActivity, bean, mParameter, proxyListener);
                currentList.add(bean);
                nativeAdMap.put(bean.adSlotId, apiNative);
            }
        }
        if (currentList.isEmpty()) {
            mHlNativeAdListener.onADError("未加载信息流自渲染广告, 无相应广告配置", InitStatus.INIT_FAIL_CODE);
            reportErrorMsg("未加载信息流自渲染广告, 无相应广告配置", InitStatus.INIT_FAIL_CODE, "初始化", null);
            return;
        }
        final Timer timer = new Timer();
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                maxTime = maxTime - 100;
                if (nativeSelectMap.size() == currentList.size()) {
                    timer.cancel();
                    formatUnifiedNativeResult(currentList);
                } else if (maxTime <= 0) {
                    timer.cancel();
                    if (!nativeSelectMap.isEmpty()) {
                        if (isLoadAd) return;
                        formatUnifiedNativeResult(currentList);
                    } else {
                        for (SlotBean bean : currentList) {
                            analyseBeanObj(bean, false, false, 0);
                        }
                        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mNativeUnifiedReqId);
                        onAdError("超时没有加载到信息流广告");
                    }
                }
            }
        };
        timer.schedule(timerTask, 0, 100);
    }

    private void formatUnifiedNativeResult(List<SlotBean> currentList) {
        String currentAdSlotId = "";
        for (Map.Entry<String, Double> entry : ecpmAdMap.entrySet()) {
            String key = entry.getKey();
            Double value = entry.getValue();
            if (value >= currentPrice) {
                currentAdSlotId = key;
                currentPrice = value;
            }
        }
        if ((currentPrice == 0 || currentPrice == 1) && !currentList.isEmpty()) {
            for (SlotBean bean : currentList) {
                Boolean aBoolean = nativeSelectMap.get(bean.adSlotId);
                if (aBoolean != null && aBoolean) {
                    currentAdSlotId = bean.adSlotId;
                    break;
                }
            }
        }
        LogUtils.e("=====untifiedsize====" + ecpmAdMap.size() + "===currentAdSlotId===" + currentAdSlotId + "====currentPrice====" + currentPrice);
        for (final SlotBean bean : currentList) {
            final String adSlotId = bean.adSlotId;
            Boolean aBoolean = nativeSelectMap.get(adSlotId);
            if (aBoolean != null && aBoolean) {
                Double ecpmNum = ecpmAdMap.get(adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                if (!isLoadAd && !TextUtils.isEmpty(currentAdSlotId) && currentAdSlotId.equals(adSlotId)) {
                    if (nativePriceAdMap != null && nativePriceAdMap.containsKey(adSlotId)) {
                        analyseBeanObj(bean, true, false, ecpmNum);
                        continue;
                    }
                    isLoadAd = true;
                    mAdFrom = bean.adFrom;
                    mActivity.runOnUiThread(() -> {
                        LogUtils.e("hlsdk====mNativeShowFrom====" + mAdFrom + "===adSlotId===" + adSlotId);
                        View view = nativeViewAdMap.get(adSlotId);
                        String adType = adSubTypeMap.get(adSlotId);
                        Float ratio = adRatioMap.get(adSlotId);
                        mHlNativeAdListener.onADLoadView(view, adType, ratio);
                    });
                    analyseBeanObj(bean, true, true, ecpmNum);
                } else {
                    analyseBeanObj(bean, true, false, ecpmNum);
                }
            } else {
                analyseBeanObj(bean, false, false, 0);
            }
        }
        LogUtils.e("====nativemPostJson===" + mPostJson + "====isLoadAd====" + isLoadAd);
        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mNativeUnifiedReqId);
        if (!isLoadAd) {
            onAdError("没有加载到信息流广告源");
        }
    }

    private void analyseBeanObj(SlotBean mBean, boolean isQuota, boolean isUse, double op) {
        try {
            JSONObject parameJson = AnalyseSlotBeanUtils.analyseBean(mBean, isQuota, isUse, op, mNativeUnifiedReqId);
            mPostJson.put(parameJson);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    private void onAdError(final String errorMsg) {
        mActivity.runOnUiThread(() -> mHlNativeAdListener.onADError(errorMsg, 100));
        reportErrorMsg(errorMsg, 100, "初始化", null);
    }

    public void loadData() {
        nativeViewAdMap = new HashMap<>();
        mPostJson = new JSONArray();
        if (nativeAdMap != null) {
            for (Map.Entry<String, INativeAd> entry : nativeAdMap.entrySet()) {
                entry.getValue().loadAd();
            }
        } else {
            mHlNativeAdListener.onADError("自渲染信息流广告load失败:", 100);
            reportErrorMsg("自渲染信息流广告load失败:", 100, "初始化", null);
        }
    }

    public double getECPM() {
        return currentPrice;
    }

    public HashMap<String, Double> getEcpmMap() {
        return ecpmAdMap;
    }

    private INativeProxyListener proxyListener = new INativeProxyListener() {
        @Override
        public void onADLoadView(View view, String adType, SlotBean slotBean, double ecpm, String adSubType,float ratio) {
            LogUtils.e("====onADLoadView===" + adType + "====adSlotId===" + slotBean.adSlotId);
            nativeSelectMap.put(slotBean.adSlotId, true);
            nativeViewAdMap.put(slotBean.adSlotId, view);
            ecpmAdMap.put(slotBean.adSlotId, ecpm);
            adSubTypeMap.put(slotBean.adSlotId, adSubType);
            adRatioMap.put(slotBean.adSlotId, ratio);
        }

        @Override
        public void onADClick(View view, SlotBean slotBean, String adSubType) {
            if (mHlNativeAdListener != null) {
                if (slotBean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(slotBean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_CLICK, mNativeUnifiedReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mActivity.runOnUiThread(() -> mHlNativeAdListener.onADClick(view, adSubType));
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mActivity.runOnUiThread(() -> mHlNativeAdListener.onADClick(view, adSubType));
                }
            }
        }

        @Override
        public void onADClose(View view) {
            isAdClose = true;
            if (mHlNativeAdListener != null) {
                mActivity.runOnUiThread(() -> mHlNativeAdListener.onADClose(view));
            }
        }

        @Override
        public void onADShow(View view, String adType, SlotBean bean, String adSubType) {
            if (isAdClose) return;
            if (mHlNativeAdListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_IMP, mNativeUnifiedReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mActivity.runOnUiThread(() -> mHlNativeAdListener.onADShow(view, adSubType));
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mActivity.runOnUiThread(() -> mHlNativeAdListener.onADShow(view, adSubType));
                }
            }
        }

        @Override
        public void onADError(String msg, int code, String adType, SlotBean slotBean) {
            LogUtils.e("====errorMsg===" + msg + "====code===" + code + "===onAdError====" + adType + "===adSlotId===" + slotBean.adSlotId);
            if (code == 102) {
                nativePriceAdMap.put(slotBean.adSlotId, slotBean);
                nativeSelectMap.put(slotBean.adSlotId, true);
            } else {
                nativeSelectMap.put(slotBean.adSlotId, false);
            }
            reportErrorMsg(msg, code, adType, slotBean);
        }
    };

    private void reportErrorMsg(String msg, int code, String type, SlotBean bean) {
        if (bean == null) {
            bean = new SlotBean();
            bean.setAppId(Config.mAppId);
            bean.setSlotId(mSlotId);
        }
        String errorMsg = type + ": errorTime==" + Utils.formatCurrentTime() + "==errorMsg:" + msg + "==errorCode==" + code;
        HlStatisManager.getInstance().reportStaticError(bean, HlStatisManager.STEPNAME_ERROR, "", mNativeUnifiedReqId, errorMsg);
    }

    public void release() {
        if (nativeAdMap != null) {
            for (Map.Entry<String, INativeAd> entry : nativeAdMap.entrySet()) {
                entry.getValue().release();
            }
        }
    }
}
