package com.halo.halojuhesdk.sdk;

import android.app.Activity;
import android.text.TextUtils;

import com.halo.halojuhesdk.core.base.utils.AnalyseSlotBeanUtils;
import com.halo.halojuhesdk.core.base.utils.LogUtils;
import com.halo.halojuhesdk.core.base.utils.Utils;
import com.halo.halojuhesdk.core.channel.IVideoProxyListener;
import com.halo.halojuhesdk.core.channel.IVideoAd;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinInsert;
import com.halo.halojuhesdk.core.channel.applovin.AppLovinReward;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiInsertAd;
import com.halo.halojuhesdk.core.channel.hlapi.HlApiRewardVideo;
import com.halo.halojuhesdk.core.common.utils.Config;
import com.halo.halojuhesdk.core.common.utils.MyUtils;
import com.halo.halojuhesdk.core.request.HlStatisManager;
import com.halo.halojuhesdk.core.request.InitDataProcessor;
import com.halo.halojuhesdk.core.request.InitStatus;
import com.halo.halojuhesdk.core.request.RequestAdManager;
import com.halo.halojuhesdk.core.request.SlotBean;
import com.halo.halojuhesdk.core.request.UpRequestCallBack;
import com.halo.halojuhesdk.sdk.listener.HlVideoListener;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Timer;
import java.util.TimerTask;
import java.util.UUID;

public class HlVideoAd {
    private final HashMap<String, SlotBean> videoPriceMap;
    private String mSlotId;
    private Map<String, Double> ecpmAdMap;
    private HlVideoListener mVideoListener;
    private Activity mActivity;
    private Map<String, Boolean> videoSelectMap;
    private Map<String, IVideoAd> videoAdMap;
    private int maxTime;
    private boolean isLoadAd;
    private boolean isAdClose;
    private JSONArray mPostJson;
    private String mShowAdSlotId;
    double currentPrice = 0;
    private String mVideoReqId = UUID.randomUUID().toString();
    public HlVideoAd(Activity context, String slotId, HlVideoListener mVideoListener) {
        MyUtils.init(context);
        mPostJson = new JSONArray();
        videoSelectMap = new HashMap<>();
        videoAdMap = new HashMap<>();
        videoPriceMap = new HashMap<>();
        ecpmAdMap = new HashMap<>();
        mActivity = context;
        mSlotId = slotId;
        maxTime = Config.AD_TIME_OUT;
        isLoadAd = false;
        isAdClose = false;
        mShowAdSlotId = "";
        this.mVideoListener = mVideoListener;
        List<SlotBean> slotBeanList = RequestAdManager.requestChannel(slotId);
        if (slotBeanList == null || slotBeanList.isEmpty()) {
            reportErrorMsg("未加载激励视频广告", InitStatus.INIT_FAIL_CODE, "初始化", null);
            mVideoListener.onAdFailed("未加载激励视频广告", InitStatus.INIT_FAIL_CODE);
        } else {
            try {
                initRewardAd(context, slotBeanList);
            } catch (Exception e) {
                e.printStackTrace();
                mVideoListener.onAdFailed("未加载激励视频广告:" + e, InitStatus.INIT_FAIL_CODE);
                reportErrorMsg("未加载激励视频广告" + e, InitStatus.INIT_FAIL_CODE, "初始化", null);
            }
        }
    }


    private void initRewardAd(Activity context, final List<SlotBean> baseBean) {
        HlStatisManager.getInstance().cVideoRequestid();
        final List<SlotBean> currentList = new ArrayList<>();
        for (int i = 0; i < baseBean.size(); i++) {
            final SlotBean bean = baseBean.get(i);
            bean.setReqId(mVideoReqId);
            String adFrom = bean.getAdFrom();
            if (adFrom.equals(InitDataProcessor.SDK_MAX) && HlAdClient.mTypeList.contains(Config.JuHeAdType.MAX)) {
                AppLovinReward appLovinVideo = new AppLovinReward(context, bean, mInnerVideoListener);
                currentList.add(bean);
                videoAdMap.put(bean.adSlotId, appLovinVideo);
            } else if (adFrom.equals(InitDataProcessor.SDK_API) && HlAdClient.mTypeList.contains(Config.JuHeAdType.API)) {
                HlApiRewardVideo apiVideo = new HlApiRewardVideo(context, bean, mInnerVideoListener);
                currentList.add(bean);
                videoAdMap.put(bean.adSlotId, apiVideo);
            }
        }
        if (currentList.isEmpty()) {
            reportErrorMsg("未加载激励视频广告, 无相应广告配置", InitStatus.INIT_FAIL_CODE, "初始化", null);
            mVideoListener.onAdFailed("未加载激励视频广告, 无相应广告配置", InitStatus.INIT_FAIL_CODE);
            return;
        }
        final Timer timer = new Timer();
        TimerTask timerTask = new TimerTask() {
            @Override
            public void run() {
                maxTime = maxTime - 100;
                if (videoSelectMap.size() == currentList.size()) {
                    timer.cancel();
                    formatVideoResult(currentList);
                } else if (maxTime <= 0) {
                    timer.cancel();
                    if (!videoSelectMap.isEmpty()) {
                        if (isLoadAd) return;
                        formatVideoResult(currentList);
                    } else {
                        for (SlotBean bean : currentList) {
                            analyseBeanObj(bean, false, false, 0);
                        }
                        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mVideoReqId);
                        onAdError("超时没有加载到视频广告");
                    }
                }
            }
        };
        timer.schedule(timerTask, 0, 100);
    }


    private void onAdError(final String errorMsg) {
        reportErrorMsg(errorMsg, 100, "广告源", null);
        mActivity.runOnUiThread(() -> {
            LogUtils.e("====error====" + errorMsg);
            mVideoListener.onAdFailed(errorMsg, 100);
        });
    }

    private void formatVideoResult(List<SlotBean> currentList) {
        String currentAdSlotId = "";
        for (Map.Entry<String, Double> entry : ecpmAdMap.entrySet()) {
            String key = entry.getKey();
            double value = entry.getValue();
            if (value >= currentPrice) {
                currentAdSlotId = key;
                currentPrice = value;
            }
        }
        if ((currentPrice == 0 || currentPrice == 1) && !currentList.isEmpty()) {
            for (SlotBean bean : currentList) {
                Boolean aBoolean = videoSelectMap.get(bean.adSlotId);
                if (aBoolean != null && aBoolean) {
                    currentAdSlotId = bean.adSlotId;
                    break;
                }
            }
        }
//        LogUtils.e("=====videosize====" + ecpmAdMap.size() + "===currentAdSlotId===" + currentAdSlotId + "====currentPrice====" + currentPrice);
        for (final SlotBean bean : currentList) {
            final String adFrom = bean.adFrom;
            final String adSlotId = bean.adSlotId;
            Boolean aBoolean = videoSelectMap.get(adSlotId);
            if (aBoolean != null && aBoolean) {
                Double ecpmNum = ecpmAdMap.get(adSlotId);
                if (ecpmNum == null) {
                    ecpmNum = 0d;
                }
                bean.setOp(ecpmNum);
                if (!isLoadAd && !TextUtils.isEmpty(currentAdSlotId) && currentAdSlotId.equals(adSlotId)) {
                    if (videoPriceMap != null && videoPriceMap.containsKey(bean.adSlotId)) {
                        analyseBeanObj(bean, true, false, ecpmNum);
                        continue;
                    }
                    isLoadAd = true;
                    mActivity.runOnUiThread(() -> {
                        mShowAdSlotId = bean.adSlotId;
                        mVideoListener.onSuccess();
                        LogUtils.e("====adFrom111====" + adFrom + "===adSlotId===" + adSlotId);
                    });
                    analyseBeanObj(bean, true, true, ecpmNum);
                } else {
                    analyseBeanObj(bean, true, false, ecpmNum);
                }
            } else {
                analyseBeanObj(bean, false, false, 0);
            }
        }
        HlStatisManager.getInstance().reportStat(mPostJson.toString(), mVideoReqId);
        if (!isLoadAd) {
            onAdError("没有加载到视频广告源");
        }
    }

    private void analyseBeanObj(SlotBean mBean, boolean isQuota, boolean isUse, double op) {
        try {
            JSONObject parameJson = AnalyseSlotBeanUtils.analyseBean(mBean, isQuota, isUse, op, mVideoReqId);
            mPostJson.put(parameJson);
        } catch (Exception e) {
            e.printStackTrace();
        }
    }

    public void loadAd() {
        mShowAdSlotId = "";
        mPostJson = new JSONArray();
        videoSelectMap = new HashMap<>();
        if (videoAdMap != null) {
            for (Map.Entry<String, IVideoAd> entry : videoAdMap.entrySet()) {
                entry.getValue().loadAd();
            }
        } else {
            reportErrorMsg("激励视频广告load失败:", 100, "广告源", null);
            mVideoListener.onAdFailed("激励视频广告load失败:", 100);
        }
    }

    public double getECPM() {
        return currentPrice;
    }

    public Map<String, Double> getEcpmMap() {
        return ecpmAdMap;
    }

    public boolean isReady() {
        return isLoadAd;
    }

    public void show() {
        if (videoAdMap != null && !TextUtils.isEmpty(mShowAdSlotId)) {
            IVideoAd iVideoAd = videoAdMap.get(mShowAdSlotId);
            if (iVideoAd != null) {
                iVideoAd.showAd();
            } else {
                mVideoListener.onAdFailed("激励视频广告load失败:", 100);
            }
        } else {
            mVideoListener.onAdFailed("激励视频广告read失败:", 100);
        }
    }

    public void onDestroy() {

    }

    private IVideoProxyListener mInnerVideoListener = new IVideoProxyListener() {

        @Override
        public void onSuccess(String adType, SlotBean slotBean, double ecpm) {
            LogUtils.e("====onSuccess===" + adType + "===ecpm===" + ecpm);
            videoSelectMap.put(slotBean.adSlotId, true);
            ecpmAdMap.put(slotBean.adSlotId, ecpm);
        }

        @Override
        public void onAdShow(final SlotBean bean) {
            if (isAdClose) return;
            if (mVideoListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_IMP, mVideoReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mActivity.runOnUiThread(() -> mVideoListener.onAdShow());
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mActivity.runOnUiThread(() -> mVideoListener.onAdShow());
                }
            }
        }

        @Override
        public void onAdClose() {
            isAdClose = true;
            if (mVideoListener != null) {
                mActivity.runOnUiThread(() -> mVideoListener.onAdClose());
            }
        }

        @Override
        public void onAdClick(final SlotBean bean) {
            if (mVideoListener != null) {
                if (bean.adFrom.contains("sdk")) {
                    HlStatisManager.getInstance().reportStaticCallBack(bean, HlStatisManager.STEPNAME_REPORT, HlStatisManager.STEPNAME_REPORT_CLICK, mVideoReqId, new UpRequestCallBack() {
                        @Override
                        public void onSuccess() {
                            mActivity.runOnUiThread(() -> mVideoListener.onAdClick());
                        }

                        @Override
                        public void onError() {

                        }
                    });
                } else {
                    mActivity.runOnUiThread(() -> mVideoListener.onAdClick());
                }
            }
        }

        //缓存失败或者播放失败统一走此处接口
        @Override
        public void onAdFailed(String msg, int code, String adType, SlotBean slotBean) {
            LogUtils.e("====error===" + adType + "===msg===" + msg + "===code===" + code + "===adSlotId===" + slotBean.adSlotId);
            if (code == 102) {
                videoPriceMap.put(slotBean.adSlotId, slotBean);
                videoSelectMap.put(slotBean.adSlotId, true);
            } else {
                videoSelectMap.put(slotBean.adSlotId, false);
            }
            reportErrorMsg(msg, code, adType, slotBean);
        }

        @Override
        public void onPlayEnd() {
            if (mVideoListener != null) {
                mActivity.runOnUiThread(() -> mVideoListener.onPlayEnd());
            }
        }

        @Override
        public void onSkipped() {
            if (mVideoListener != null) {
                mActivity.runOnUiThread(() -> mVideoListener.onSkipped());
            }
        }

        @Override
        public void onRewardArrived() {
            if (mVideoListener != null) {
                mActivity.runOnUiThread(() -> mVideoListener.onRewardArrived());
            }
        }
    };

    private void reportErrorMsg(String msg, int code, String type, SlotBean bean) {
        if (bean == null) {
            bean = new SlotBean();
            bean.setAppId(Config.mAppId);
            bean.setSlotId(mSlotId);
        }
        String errorMsg = type + ": errorTime==" + Utils.formatCurrentTime() + "==errorMsg:" + msg + "==errorCode==" + code;
        HlStatisManager.getInstance().reportStaticError(bean, HlStatisManager.STEPNAME_ERROR, "", mVideoReqId, errorMsg);
    }
}
